from collections import namedtuple

__all__ = ['tarjan',
           'tarjan_iter',
           'tarjan_recursive']

""" Context used to implement the algorithm without recursion in @tarjan
    and @tarjan_iter """
TarjanContext = namedtuple('TarjanContext',
                                ['g',           # the graph
                                 'S',           # The main stack of the alg.
                                 'S_set',       # == set(S) for performance
                                 'index',       # { v : <index of v> }
                                 'lowlink',     # { v : <lowlink of v> }
                                 'T',           # stack to replace recursion
                                 'ret'])        # return code

def _tarjan_head(ctx, v):
        """ Used by @tarjan and @tarjan_iter.  This is the head of the
            main iteration """
        ctx.index[v] = len(ctx.index)
        ctx.lowlink[v] = ctx.index[v]
        ctx.S.append(v)
        ctx.S_set.add(v)
        it = iter(ctx.g.get(v, ()))
        ctx.T.append((it,False,v,None))

def _tarjan_body(ctx, it, v):
        """ Used by @tarjan and @tarjan_iter.  This is the body of the
            main iteration """
        for w in it:
                if w not in ctx.index:
                        ctx.T.append((it,True,v,w))
                        _tarjan_head(ctx, w)
                        return
                if w in ctx.S_set:
                        ctx.lowlink[v] = min(ctx.lowlink[v], ctx.index[w])
        if ctx.lowlink[v] == ctx.index[v]:
                scc = []
                w = None
                while v != w:
                        w = ctx.S.pop()
                        scc.append(w)
                        ctx.S_set.remove(w)
                ctx.ret.append(scc)

def tarjan_iter(g):
        """ Returns the strongly connected components of the graph @g
            in a topological order.

                @g is the graph represented as a dictionary
                        { <vertex> : <successors of vertex> }.

            This function does not recurse.  It returns an iterator. """
        ctx = TarjanContext(
                g = g,
                S = [],
                S_set = set(),
                index = {},
                lowlink = {},
                T = [],
                ret = [])
        main_iter = iter(g)
        while True:
                try:
                        v = next(main_iter)
                except StopIteration:
                        return
                if v not in ctx.index:
                        _tarjan_head(ctx, v)
                while ctx.T:
                        it, inside, v, w = ctx.T.pop()
                        if inside:
                                ctx.lowlink[v] = min(ctx.lowlink[w],
                                                        ctx.lowlink[v])
                        _tarjan_body(ctx, it, v)
                        if ctx.ret:
                                assert len(ctx.ret) == 1
                                yield ctx.ret.pop()

def tarjan(g):
        """ Returns the strongly connected components of the graph @g
            in a topological order.

                @g is the graph represented as a dictionary
                        { <vertex> : <successors of vertex> }.
        
            This function does not recurse. """
        ctx = TarjanContext(
                g = g,
                S = [],
                S_set = set(),
                index = {},
                lowlink = {},
                T = [],
                ret = [])
        main_iter = iter(g)
        while True:
                try:
                        v = next(main_iter)
                except StopIteration:
                        return ctx.ret
                if v not in ctx.index:
                        _tarjan_head(ctx, v)
                while ctx.T:
                        it, inside, v, w = ctx.T.pop()
                        if inside:
                                ctx.lowlink[v] = min(ctx.lowlink[w],
                                                        ctx.lowlink[v])
                        _tarjan_body(ctx, it, v)

def tarjan_recursive(g):
        """ Returns the strongly connected components of the graph @g
            in a topological order.

                @g is the graph represented as a dictionary
                        { <vertex> : <successors of vertex> }.
                        
            This function recurses --- large graphs may cause a stack
            overflow. """
        S = []
        S_set = set()
        index = {}
        lowlink = {}
        ret = []

        def visit(v):
                index[v] = len(index)
                lowlink[v] = index[v]
                S.append(v)
                S_set.add(v)
                for w in g.get(v,()):
                        if w not in index:
                                visit(w)
                                lowlink[v] = min(lowlink[w], lowlink[v])
                        elif w in S_set:
                                lowlink[v] = min(lowlink[v], index[w])
                if lowlink[v] == index[v]:
                        scc = []
                        w = None
                        while v != w:
                                w = S.pop()
                                scc.append(w)
                                S_set.remove(w)
                        ret.append(scc)

        for v in g:
                if not v in index:
                        visit(v)
        return ret



import unittest
import tarjan.tc

class TestMain(unittest.TestCase):
    def test_mainExample(self):
        # see doc/example.png
        g = {1:[2],2:[1,5],3:[4],4:[3,5], 5:[6],6:[7],7:[8],8:[6,9],9:[]}
        res = tarjan.tc.tc(g)
        res = dict([(k, frozenset(res[k])) for k in res])
        self.assertEqual(res, {1: frozenset([1, 2, 5, 6, 7, 8, 9]),
                               2: frozenset([1, 2, 5, 6, 7, 8, 9]),
                               3: frozenset([3, 4, 5, 6, 7, 8, 9]),
                               4: frozenset([3, 4, 5, 6, 7, 8, 9]),
                               5: frozenset([8, 9, 6, 7]),
                               6: frozenset([8, 9, 6, 7]),
                               7: frozenset([8, 9, 6, 7]),
                               8: frozenset([8, 9, 6, 7]),
                               9: frozenset()})

if __name__ == '__main__':
    unittest.main()



import unittest
import tarjan

class TestMain(unittest.TestCase):
    def test_mainExample(self):
        # see doc/example.png
        g = {1:[2],2:[1,5],3:[4],4:[3,5], 5:[6],6:[7],7:[8],8:[6,9],9:[]}
        for f in (tarjan.tarjan, tarjan.tarjan_recursive):
            res = list(map(frozenset, f(g)))
            self.assertTrue(res == list(map(frozenset,
                                [[9], [8,7,6], [5], [2,1], [4,3]])) or
                            res == list(map(frozenset,
                                [[9], [8,7,6], [5], [4,3], [2,1]])))

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

from setuptools import setup

setup(name='tarjan',
      version='0.2.3.1',
      description='Implementation of Tarjan\'s algorithm: resolve cyclic deps',
      long_description='{0:s}\n{1:s}'.format(
          open('README.rst').read(),
          open('CHANGES.rst').read()),
      author='Bas Westerbaan',
      author_email='bas@westerbaan.name',
      url='http://github.com/bwesterb/py-tarjan/',
      packages=['tarjan', 'tarjan.tests'],
      zip_safe=True,
      package_dir={'tarjan': 'src'},
      test_suite='tarjan.tests',
      classifiers=[
          'Development Status :: 5 - Production/Stable',
          'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
          'Programming Language :: Python :: 2.6',
          'Programming Language :: Python :: 2.7',
          'Programming Language :: Python :: 3.2',
          'Programming Language :: Python :: 3.3',
          'Programming Language :: Python :: 3.4',
          'Programming Language :: Python :: 3.5',
          ],
      )



from tarjan import tarjan

def tc(g):
        """ Given a graph @g, returns the transitive closure of @g """
        ret = {}
        for scc in tarjan(g):
                ws = set()
                ews = set()
                for v in scc:
                        ws.update(g[v])
                for w in ws:
                        assert w in ret or w in scc
                        ews.add(w)
                        ews.update(ret.get(w,()))
                if len(scc) > 1:
                        ews.update(scc)
                ews = tuple(ews)
                for v in scc:
                        ret[v] = ews
        return ret
